#include <functional>
#include <iostream>
#include <set>
#include <vector>
#include <algorithm>
#include <cctype>
#include <sstream>
#include <map>

static int numbers[] = { 
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 
  10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 
  20, 22, 30, 33, 
  40, 44, 50, 55,
  60, 66, 70, 77, 
  80, 88, 90, 99,
  100, 101, 111, 123, 
  200, 202, 222, 234, 
  300, 303, 333, 345, 
  400, 404, 444, 456,
  500, 505, 555, 567,
  600, 606, 666, 678,
  700, 707, 777, 789,
  800, 808, 888, 
  900, 909, 999,
  1000
};

template<typename T>
void make_shorts(T& out)
{
  std::cerr << "Creating short word list..." << std::endl;

  for (unsigned n = 0; n < sizeof(numbers)/sizeof(numbers[0]); n++)
    {
      std::ostringstream s;
      s << numbers[n];
      out.insert(s.str());
    }
}

template<typename T>
void read_input(unsigned n, T& words)
{
  std::cerr << "Reading and filtering input word list..." << std::endl;

  unsigned count = 0;

  while (std::cin)
    {
      std::string w;
      std::cin >> w;
      if (w.size() > 2 &&
	  find_if(w.begin(), w.end(), 
		  std::not1(std::ptr_fun(islower))) == w.end())
	{
	  words[w.size()].insert(w);
	  ++count;
	}
    }

  if (count < n)
    {
      std::cerr << "Input list has too few words (must have more than "
		<< n << " words)." << std::endl;
      exit(1);
    }

}

template<typename T, typename U>
void select_words(unsigned n, T& selected, U& words)
{
  int l = 3;
  while (selected.size() < n)
    {
      std::cerr << "Extracting " << l << "-characters long words..." 
		<< std::endl;
      selected.insert(selected.end(), words[l].begin(), words[l].end());
      ++l;
    }
}

template<typename T, typename U>
void shuffle_extract(unsigned n, T& words, U& selected)
{
  std::cerr << "Shuffling selected list..." << std::endl;
  std::random_shuffle(selected.begin(), selected.end());

  std::cerr << "Extracting words..." << std::endl;
  while (n)
      words.insert(selected[n--]);
}

template<typename T>
void spit_output(const T& l)
{ 
  for (typename T::const_iterator i = l.begin();
       i != l.end(); ++i)
    std::cout << *i << std::endl;
}

int main()
{
  std::set<std::string> shorts;
  make_shorts(shorts);

  unsigned n = 4096 - shorts.size();

  std::map<unsigned, std::set<std::string> > words;
  read_input(n, words);

  std::vector<std::string> selected;
  select_words(n, selected, words);

  words.clear();
  shuffle_extract(n, words[0], selected);

  spit_output(shorts);
  spit_output(words[0]);

  return 0;
}

