#! /bin/sh

# ppgen.sh - Passphrase Generator
#
# Copyright (C) 2003 Raphael Poss <ppgen@raphael.poss.name>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#

PROPOSITIONS=/usr/share/ppgen/propositions
INTRO=/usr/share/ppgen/intro.txt
LISTDIR=/usr/share/ppgen/lists

#################
## Initialize

trap 'stop' SIGTERM SIGQUIT SIGINT

opie-sh -t "Passphrase generator" -f "$INTRO" >/dev/null 2>&1 &
w=$!
sleep 2

stop() {
  kill $w
  [ "x$T" = "x" ] || rm -f "$T"
  exit 0
}

die() {
  opie-sh -m -t "ppgen" -e -M "$*" -g 2>/dev/null
  stop
}

###############################################
## Look for word lists and ask to choose a list

T=`mktemp /tmp/ppgen-XXXXXXXX`
[ -w "$T" ] || die "Initialization error."
( cd "$LISTDIR"; ls ) >>"$T"

[ `wc -l <"$T"` = 0 ] && die "No word lists available."

list=`opie-sh -i -t "ppgen" -L "Choose a word list:" -l -F "$T" -g 2>/dev/null`

[ "x$list" = "x" ] && die "No word list selected."
[ -r "$LISTDIR/$list" ] || die "<h3>$LISTDIR/$list</h3> is not readable."
[ `wc -l <"$LISTDIR/$list"` = 4096 ] || die "<h3>$LISTDIR/$list</h3> does not contain 4096 entries."

##########################
## Ask for number of words

nrwords=`opie-sh -i -t "ppgen" -L "Number of words:" -l -F "$PROPOSITIONS" -g 2>/dev/null`

[ "x$nrwords" = "x" ] && die "No number of words selected."
expr "$nrwords" : "[0-9]*$" >/dev/null 2>&1 || die "$nrwords: Invalid number of words."

finish=no
while [ "$finish" = "no" ]; do

    qcop QPE/System "busy()" &

    #####################
    ## Compute passphrase

    l=''

    while [ `echo "$l" | wc -c` -lt `expr $nrwords \* 3 + 2` ]; do 

	l=''

	pairs=`expr $nrwords / 2 + 1`
	rands=`ppgen-rand $pairs 3`
	n=$nrwords
	for r in $rands; do

	  [ $n = 0 ] && break

	  p=`expr $r / 4096 + 1`
	  l="$l"`sed -n -e "$p{p;}" <"$LISTDIR/$list"|tr '\n' ' '`

	  [ $n = 1 ] && break

	  p=`expr $r % 4096 + 1`
	  l="$l"`sed -n -e "$p{p;}" <"$LISTDIR/$list"|tr '\n' ' '`

	  n=`expr $n - 2`
	done

    done

    #####################
    ## Display passphrase

    msg="<h3>Passphrase:</h3>
    $l"

    qcop QPE/System "notBusy(QString)" "done." &

    opie-sh -m -t "ppgen" -M "$msg" -0 "Another!" -1 "Quit" -g >/dev/null 2>&1

    res=$?
    [ $res = 0 ] || finish=yes

done

stop

